/*! For license information please see 848.0c8ba387ba66a5249ba1.js.LICENSE.txt */
"use strict";(self.webpackChunkjupytutor=self.webpackChunkjupytutor||[]).push([[86,467,848],{1020(e,r,t){var o=t(3345),n=Symbol.for("react.element"),f=Symbol.for("react.fragment"),u=Object.prototype.hasOwnProperty,p=o.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,s={key:!0,ref:!0,__self:!0,__source:!0};function _(e,r,t){var o,f={},_=null,a=null;for(o in void 0!==t&&(_=""+t),void 0!==r.key&&(_=""+r.key),void 0!==r.ref&&(a=r.ref),r)u.call(r,o)&&!s.hasOwnProperty(o)&&(f[o]=r[o]);if(e&&e.defaultProps)for(o in r=e.defaultProps)void 0===f[o]&&(f[o]=r[o]);return{$$typeof:n,type:e,key:_,ref:a,props:f,_owner:p.current}}r.Fragment=f,r.jsx=_,r.jsxs=_},4848(e,r,t){e.exports=t(1020)}}]);