"use strict";(self.webpackChunkjupytutor=self.webpackChunkjupytutor||[]).push([[660],{7660(n,t,e){e.r(t),e.d(t,{CONTINUE:()=>f,EXIT:()=>c,SKIP:()=>l,visit:()=>p});const r=function(n){if(null==n)return i;if("function"==typeof n)return o(n);if("object"==typeof n)return Array.isArray(n)?function(n){const t=[];let e=-1;for(;++e<n.length;)t[e]=r(n[e]);return o(function(...n){let e=-1;for(;++e<t.length;)if(t[e].apply(this,n))return!0;return!1})}(n):function(n){const t=n;return o(function(e){const r=e;let o;for(o in n)if(r[o]!==t[o])return!1;return!0})}(n);if("string"==typeof n)return t=n,o(function(n){return n&&n.type===t});var t;throw new Error("Expected function, string, or object as test")};function o(n){return function(t,e,r){return Boolean(function(n){return null!==n&&"object"==typeof n&&"type"in n}(t)&&n.call(this,t,"number"==typeof e?e:void 0,r||void 0))}}function i(){return!0}const u=[],f=!0,c=!1,l="skip";function p(n,t,e,o){let i,p,s;"function"==typeof t&&"function"!=typeof e?(p=void 0,s=t,i=e):(p=t,s=e,i=o),function(n,t,e,o){let i;"function"==typeof t&&"function"!=typeof e?(o=e,e=t):i=t;const p=r(i),s=o?-1:1;!function n(r,i,y){const a=r&&"object"==typeof r?r:{};if("string"==typeof a.type){const n="string"==typeof a.tagName?a.tagName:"string"==typeof a.name?a.name:void 0;Object.defineProperty(d,"name",{value:"node ("+r.type+(n?"<"+n+">":"")+")"})}return d;function d(){let a,d,h,g=u;if((!t||p(r,i,y[y.length-1]||void 0))&&(g=function(n){return Array.isArray(n)?n:"number"==typeof n?[f,n]:null==n?u:[n]}(e(r,y)),g[0]===c))return g;if("children"in r&&r.children){const t=r;if(t.children&&g[0]!==l)for(d=(o?t.children.length:-1)+s,h=y.concat(t);d>-1&&d<t.children.length;){const e=t.children[d];if(a=n(e,d,h)(),a[0]===c)return a;d="number"==typeof a[1]?a[1]:d+s}}return g}}(n,void 0,[])()}(n,p,function(n,t){const e=t[t.length-1],r=e?e.children.indexOf(n):void 0;return s(n,r,e)},i)}}}]);